<?php

class BWFAN_WC_Customer_Phone extends BWFAN_Merge_Tag {

	private static $instance = null;

	public function __construct() {
		$this->tag_name        = 'customer_phone';
		$this->tag_description = __( 'Customer Phone', 'wp-marketing-automations' );
		add_shortcode( 'bwfan_customer_phone', array( $this, 'parse_shortcode' ) );
	}

	public static function get_instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Parse the merge tag and return its value.
	 *
	 * @param $attr
	 *
	 * @return mixed|string|void
	 */
	public function parse_shortcode( $attr ) {
		$get_data = BWFAN_Merge_Tag_Loader::get_data();
		if ( true === $get_data['is_preview'] ) {
			return $this->get_dummy_preview();
		}

		$customer_phone = '';
		if ( isset( $get_data['phone'] ) && ! empty( $get_data['phone'] ) ) {
			$customer_phone = $get_data['phone'];
		} elseif ( isset( $get_data['wc_order'] ) && ! empty( $get_data['wc_order'] ) ) {
			$customer_phone = BWFAN_Woocommerce_Compatibility::get_order_data( $get_data['wc_order'], '_billing_phone' );
		} elseif ( isset( $get_data['user_id'] ) && ! empty( $get_data['user_id'] ) ) {
			$customer_phone = get_user_meta( $get_data['user_id'], 'billing_phone', true );
		} elseif ( isset( $get_data['cart_details'] ) && ! empty( $get_data['cart_details'] ) ) {
			$data = json_decode( $get_data['cart_details']['checkout_data'], true );
			if ( isset( $data['fields'] ) && isset( $data['fields']['billing_phone'] ) ) {
				$customer_phone = $data['fields']['billing_phone'];
			}
		}

		return $this->parse_shortcode_output( $customer_phone, $attr );
	}

	/**
	 * Show dummy value of the current merge tag.
	 *
	 * @return string
	 */
	public function get_dummy_preview() {
		return '9999888777';
	}


}

/**
 * Register this merge tag to a group.
 */
if ( bwfan_is_woocommerce_active() ) {
	BWFAN_Merge_Tag_Loader::register( 'wc_customer', 'BWFAN_WC_Customer_Phone' );
}

